# https://developers.notion.com/reference/block#template
from dataclasses import dataclass, field
from typing import List, Optional

from htmlBuilder.tags import Div, HtmlTag

from unstructured_ingest.connector.notion.interfaces import BlockBase
from unstructured_ingest.connector.notion.types.rich_text import RichText


@dataclass
class Template(BlockBase):
    children: List[dict] = field(default_factory=list)
    rich_text: List[RichText] = field(default_factory=list)

    @staticmethod
    def can_have_children() -> bool:
        return True

    @classmethod
    def from_dict(cls, data: dict):
        rich_text = data.pop("rich_text", [])
        template = cls(**data)
        template.rich_text = [RichText.from_dict(rt) for rt in rich_text]
        return template

    def get_html(self) -> Optional[HtmlTag]:
        if not self.rich_text:
            return None
        return Div([], [rt.get_html() for rt in self.rich_text])



import json
import os
from pathlib import Path

import pytest

from test.integration.embedders.utils import (
    validate_embedding_output,
    validate_raw_embedder,
    validate_raw_embedder_async,
)
from test.integration.utils import requires_env
from unstructured_ingest.embed.openai import (
    AsyncOpenAIEmbeddingEncoder,
    OpenAIEmbeddingConfig,
    OpenAIEmbeddingEncoder,
)
from unstructured_ingest.v2.errors import UserAuthError
from unstructured_ingest.v2.processes.embedder import Embedder, EmbedderConfig

API_KEY = "OPENAI_API_KEY"


def get_api_key() -> str:
    api_key = os.getenv(API_KEY, None)
    assert api_key
    return api_key


@requires_env(API_KEY)
def test_openai_embedder(embedder_file: Path):
    api_key = get_api_key()
    embedder_config = EmbedderConfig(embedding_provider="openai", embedding_api_key=api_key)
    embedder = Embedder(config=embedder_config)
    results = embedder.run(elements_filepath=embedder_file)
    assert results
    with embedder_file.open("r") as f:
        original_elements = json.load(f)
    validate_embedding_output(original_elements=original_elements, output_elements=results)


@requires_env(API_KEY)
def test_raw_openai_embedder(embedder_file: Path):
    api_key = get_api_key()
    embedder = OpenAIEmbeddingEncoder(
        config=OpenAIEmbeddingConfig(
            api_key=api_key,
        )
    )
    validate_raw_embedder(embedder=embedder, embedder_file=embedder_file, expected_dimension=1536)


def test_raw_openai_embedder_invalid_credentials():
    embedder = OpenAIEmbeddingEncoder(
        config=OpenAIEmbeddingConfig(
            api_key="fake_api_key",
        )
    )
    with pytest.raises(UserAuthError):
        embedder.get_exemplary_embedding()


@requires_env(API_KEY)
@pytest.mark.asyncio
async def test_raw_async_openai_embedder(embedder_file: Path):
    api_key = get_api_key()
    embedder = AsyncOpenAIEmbeddingEncoder(
        config=OpenAIEmbeddingConfig(
            api_key=api_key,
        )
    )
    await validate_raw_embedder_async(
        embedder=embedder, embedder_file=embedder_file, expected_dimension=1536
    )



from unstructured_ingest.embed.openai import OpenAIEmbeddingConfig, OpenAIEmbeddingEncoder


def test_embed_documents_does_not_break_element_to_dict(mocker):
    # Mocked client with the desired behavior for embed_documents
    raw_elements = [{"text": f"This is sentence {i + 1}"} for i in range(4)]
    mock_response = mocker.MagicMock()
    mock_response_data = []
    for i in range(2):
        mock_response_d = mocker.MagicMock()
        mock_response_d.embedding = [1, 2]
        mock_response_data.append(mock_response_d)
    mock_response.data = mock_response_data
    mock_client = mocker.MagicMock()
    mock_client.embeddings.create.return_value = mock_response

    # Mock get_client to return our mock_client
    mocker.patch.object(OpenAIEmbeddingConfig, "get_client", return_value=mock_client)

    encoder = OpenAIEmbeddingEncoder(config=OpenAIEmbeddingConfig(api_key="api_key", batch_size=2))

    elements = encoder.embed_documents(
        elements=raw_elements,
    )
    assert len(elements) == 4
    assert elements[0]["text"] == "This is sentence 1"
    assert elements[1]["text"] == "This is sentence 2"
    assert mock_client.embeddings.create.call_count == 2
